<?php

/*
Plugin Name: Demandbase
Plugin URI: http://support.demandbase.com
Description: Plugin for managing Demandbase content on a WordPress site
Author: Anthony Palazzetti
Version: 2.2.4
Author URI: http://www.demandbase.com
*/

class DemandbaseCore {

    /**
     * DemandbaseAPI class instance
     *
     * @var stdClass
     */
    protected $demandbase_api;

    /**
     * Cache the company profile here to eliminate multiple API calls
     *
     * @var Array
     */
    protected $company_profile;

    /**
     * Setup this class when instantiated
     *
     */
    public function __construct()
    {
        include 'libraries/Demandbase.php';

        $this->demandbase_api = new Demandbase();

        add_action('admin_init', array($this, 'register_javascript'));
        add_action('admin_menu', array($this, 'menu_links'));
        add_action( 'add_meta_boxes', array($this, 'add_custom_box'));
        add_shortcode('db_attribute', array($this, 'attribute_shortcodes'));
        add_shortcode('db_conditional', array($this, 'conditional_shortcodes'));
        add_filter('widget_text', 'do_shortcode');

        // Cache the company profile
        $this->company_profile = $this->demandbase_api->get_company_profile();
    }

    /**
     * Tell WP what JavaScript files we need for our plugin
     *
     * @param void
     * @return void
     */
    public function register_javascript ()
    {
        wp_enqueue_script('jquery');

        wp_register_script( 'db_main', plugins_url('demandbase-content-module/assets/js/db_main.js'));
        wp_enqueue_script('db_main');
    }


    /**
     * Generate admin menu links in sidebar
     *
     * @param void
     * @return void
     */
    public function menu_links ()
    {
        add_menu_page("demandbase", "Demandbase", 1, "demandbase", array($this, "settings"));
        add_submenu_page("demandbase", "How To", "How To", 1, "demandbase-howto", array($this, "howto"));
        add_submenu_page("demandbase", "Attributes", "Attributes", 1, "demandbase-entitlements", array($this, "entitlements"));
    }

    /**
     * Generate settings page and hande POSTing
     *
     * @param void
     * @return void
     */
    public function settings ()
    {
        $db_settings = $this->demandbase_api->get_config();

        // The form below posts to the same URL
        if($_POST['demandbase_hidden'] == 'true') {

            include_once 'libraries/Form_validation.php';

            $form_validation = new Form_validation();

            $errors = $form_validation->validate_post();

            if(empty($errors))
            {
                // Write values to database
                $config = array(
                    'demandbase_api_key' => $_POST['demandbase_api_key'],
                    'demandbase_company_ip' => $_POST['demandbase_company_ip'],
                    'demandbase_sim_active' => $_POST['demandbase_sim_active'],
                    'demandbase_custom_ip_active' => $_POST['demandbase_custom_ip_active'],
                    'demandbase_custom_ip' => $_POST['demandbase_custom_ip']
                    );

                $this->demandbase_api->save_config($config);

                echo '<script>setTimeout(function () { window.location.href = "'.$_SERVER['REQUEST_URI'].'"; }, 1000); </script>';
                echo '<div class="updated"><p>Your changes have been updated. You will be redirected.</p></div>';

                return true;
            }
            else
            {
                include_once('views/fragments/error_messages.php');
            }
        }

        include_once('views/pages/settings.php');
    }


    /**
     * Generate how-to page and provide settings to it
     *
     * @param void
     * @return void
     */
    function howto() {

        $db_settings = $this->demandbase_api->get_config();

        include('views/pages/howto.php');
    }


    /**
     * Generate entitlements page and provide settings to it
     *
     * @param void
     * @return void
     */
    function entitlements() {

        $db_settings = $this->demandbase_api->get_config();
        $entitlements = $this->demandbase_api->get_company_profile(FALSE, '34.145.115.54');

        include('views/pages/entitlements.php');
    }


    /**
     * Generate attribute shortcodes
     * This specific shortcode is for echoing visitor attributes
     *
     * Usage: [db_attribute attr="marketing_alias"]
     *
     * @param string $atts Attributes sent by WP automatically
     * @return string $response Attribute wrapped the way we found it
     */
    function attribute_shortcodes ($atts) {
        extract(shortcode_atts(array('attr' => ''), $atts ));
        return $this->company_profile["{$attr}"];
    }


    /**
     * Generate attribute shortcodes
     * This specific shortcode is for echoing vositor attributes
     *
     * Usage: [db_attribute attr="marketing_alias"]
     *
     * @param string $atts Attributes sent by WP automatically
     * @param string $content The content inside the shortcode wrappers
     * @return string $response Returns the string enclosed in the shortcodes
     */
    function conditional_shortcodes ($atts, $content = null) {

        // If there is no shortcode for db_conditional, return here
        if(shortcode_exists('db_conditional') === false) return;

        // Grab the company profile
        $companyProfile = $this->company_profile;

        // Lowercase all values
        $companyProfile = array_map('strtolower', $companyProfile);

        // Make array with only the keys set (no values so wordpress can
        // fill in properly)
        $companyProfileKeys = array_fill_keys(array_keys($companyProfile), "");
        $companyProfileKeys['not'] = false;

        // Wordpress populates the array with the values extracted from the
        // shortcodes
        $attributeProfile = shortcode_atts($companyProfileKeys, $atts);

        // If not="not" in the shortcode, change it to true
        if($attributeProfile['not'] == 'not') $attributeProfile['not'] = true;
        else $attributeProfile['not'] = false;

        $matchOccurred = false;

        // Iterate over the values we have in the attribute profile and check
        // against the company profile for matches. If we have a match, return
        // the content.
        foreach ($attributeProfile as $key => $value) {

            // If we have an empty value or a not flag, skip
            if($key === 'not' OR empty($value)) continue;

            $value = strtolower($value);
            $value = html_entity_decode($value);

            // If we have a comma in the value, we are dealing with multiple
            // values and need to check each accordingly
            if(strpos($value, ',') !== false)
            {
                // Split into array of values and trim strings
                $value = array_map('trim', explode(',', $value));
            }

            // Check for matches. If we have an array, check each value in
            // array
            if(is_array($value))
            {
                foreach ($value as $sub_val) {

                    if($companyProfile[$key] === $sub_val)
                    {
                        $matchOccurred = true;
                    }
                }
            }
            else
            {
                if($companyProfile[$key] === $value)
                {
                    $matchOccurred = true;
                }
            }
        }

        if($matchOccurred === true AND $attributeProfile['not'] === false)
        {
            return do_shortcode($content);
        }
        if($matchOccurred === false AND $attributeProfile['not'] === true)
        {
            return do_shortcode($content);
        }
    }


    /**
     * Adds Demandbase shortcode widget to page and post editing
     *
     * @param void
     * @return void
     */
    function add_custom_box () {

        $screens = array( 'post', 'page' );

        foreach ( $screens as $screen ) {
            add_meta_box('demandbase_sectionid','Demandbase Shortcode Builder', array($this, 'shortcodes'), $screen);
        }
    }

    /**
     * Content for shortcode builder
     *
     * @param void
     * @return void
     */
    function shortcodes() {

        $entitlements = $this->demandbase_api->get_company_profile(FALSE, '34.145.115.54');

        include('views/widgets/shortcode_helper.php');
    }

}

$demandbase_core = new DemandbaseCore();
