<?php

class Demandbase
{

    /**
     * The Demandbase API URL
     *
     * @var string
     */
    protected $demandbase_url = 'http://api.demandbase.com/api/v2/ip.json';


    /**
     * The profile of the currently identified company
     *
     * @var Array
     */
    protected $company_profile;


    public function __construct()
    {
        $this->company_profile = null;
    }


    /**
     * Grab Demandbase settings from database
     *
     * @param void
     * @return array $db_settings
     */
    public function get_config ()
    {
        $db_settings = array(
            'api_key'               => get_option('demandbase_api_key'), // Client's entered API key
            'simulation_active'     => get_option('demandbase_sim_active'), // Is sim mode actve?
            'test_company_ip'       => get_option('demandbase_company_ip'), // Testing company IP
            'custom_ip_active'      => get_option('demandbase_custom_ip_active'), // Custom IP mode is active
            'test_custom_ip'        => get_option('demandbase_custom_ip') // Custom test IP
            );

        return $db_settings;
    }


    /**
     * Save Demandbase settings to database
     *
     * @param array $config Array of values
     * @return boolean
     */
    public function save_config ($config = array())
    {
        update_option('demandbase_api_key', $config['demandbase_api_key']);
        update_option('demandbase_company_ip', $config['demandbase_company_ip']);
        update_option('demandbase_sim_active', $config['demandbase_sim_active']);
        update_option('demandbase_custom_ip_active', $config['demandbase_custom_ip_active']);
        update_option('demandbase_custom_ip', $config['demandbase_custom_ip']);

        return TRUE;
    }


    /**
     * Flatten multidimensional arrays
     *
     * @param array $array The array to flatten
     * @param string $prefix String to prefic the returned array with
     * @return array $result The flattened array
     */
    public function flatten ($array = array(), $prefix = '') {

        $result = array();
        foreach($array as $key => $value) {
            if(is_array($value)) {
                $result = $result + $this->flatten($value, $prefix . $key . '_');
            }
            else {
                $result[$prefix . $key] = $value;
            }
        }
        return $result;
    }


    /**
     * Get visitor IP
     * @return string    $ip
     */
    public function getUserIP()
    {
        $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
        $client  = @$_SERVER['HTTP_CLIENT_IP'];
        $remote  = $_SERVER['REMOTE_ADDR'];

        if(filter_var($forward, FILTER_VALIDATE_IP)) $ip = $forward;
        elseif(filter_var($client, FILTER_VALIDATE_IP)) $ip = $client;
        else $ip = $remote;

        return $ip;
    }


    /**
     * Make Demandbase API call
     *
     * @param string    $api_key - Demandbase API key to override the one in config
     * @param string    $ip_address - IP Address to override detecting the users
     * @return array    $response - Flattened company profile
     */
    public function get_company_profile($api_key = FALSE, $ip_address = FALSE)
    {
        // If the caller of this function did not specify an API key or IP
        // address, try using the company profile we already have saved
        // (if any)
        if($api_key === FALSE AND $ip_address === FALSE)
        {
            if($this->company_profile !== null) return $this->company_profile;
        }

        $db_settings = $this->get_config();

        $api_key = empty($api_key) ? $db_settings['api_key'] : $api_key;
        $ip_address = empty($ip_address) ? $this->getUserIP() : $ip_address;

        // If simulation mode is active, get the test ip address from database
        if($db_settings['simulation_active'] == 1) {
            if($db_settings['custom_ip_active'] == 1)
            {
                $ip_address = $db_settings['test_custom_ip'];
            } else {
                $ip_address = $db_settings['test_company_ip'];
            }
        }

        // Build query string for the API call
        $url = $this->demandbase_url.'?'.'key='.$api_key.'&query='.$ip_address;

        // Response from the Demandbase API
        $response = wp_remote_get( $url, array('timeout' => 2) );

        // If wp_remote_get reported errors, return false
        if(isset($response->errors)) return [];

        if(empty(json_decode($response['body'], TRUE))) return [];

        // Return the flattened company profile
        return $this->flatten(json_decode($response['body'], TRUE));
    }

}
